@@ -1,5 +1,5 @@
 #
-# Copyright 2020 Nebulon, Inc.
+# Copyright 2021 Nebulon, Inc.
 # All Rights Reserved.
 #
 # DISCLAIMER: THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND,
@@ -11,14 +11,28 @@
 # DEALINGS IN THE SOFTWARE.
 #
 
+from time import sleep
+from datetime import datetime
 from .graphqlclient import GraphQLParam, NebMixin
 from .common import PageInput, read_value
-from .filters import UuidFilter, IntFilter
+from .filters import UUIDFilter, IntFilter, StringFilter
 from .sorting import SortDirection
+from .recipe import RecipeState, NPodRecipeFilter
+from .npods import _TIMEOUT_SECONDS
 from .tokens import TokenResponse
 
+__all__ = [
+    "LUN",
+    "LUNSort",
+    "LUNsMixin",
+    "LUNList",
+    "LUNFilter",
+    "CreateLUNInput",
+    "BatchDeleteLUNInput",
+]
 
-class LunSort:
+
+class LUNSort:
     """A sort object for LUNs
 
     Allows sorting LUNs on common properties. The sort object allows
@@ -49,7 +63,7 @@ class LunSort:
         return result
 
 
-class LunFilter:
+class LUNFilter:
     """A filter object to filter LUNs.
 
     Allows filtering for specific LUNs. The filter allows only one property to
@@ -59,8 +73,12 @@ class LunFilter:
 
     def __init__(
             self,
-            uuid: UuidFilter = None,
+            uuid: UUIDFilter = None,
             lun_id: IntFilter = None,
+            spu_serial: StringFilter = None,
+            volume_uuid: UUIDFilter = None,
+            npod_uuid: UUIDFilter = None,
+            host_uuid: StringFilter = None,
             and_filter=None,
             or_filter=None
     ):
@@ -71,22 +89,38 @@ class LunFilter:
         options to concatenate multiple filters.
 
         :param uuid: Filter based on datacenter unique identifiers
-        :type uuid: UuidFilter, optional
+        :type uuid: UUIDFilter, optional
         :param lun_id: Filter based on ``lun_id`` property of a LUN
         :type lun_id: IntFilter, optional
+        :param spu_serial: Filter based on the SPU serial number on which the
+            LUN is active
+        :type spu_serial: StringFilter, optional
+        :param volume_uuid: Filter based on the unique identifier of the
+            volume this LUN relates to
+        :type volume_uuid: UUIDFilter, optional
+        :param npod_uuid: Filter based on the nPod unique identifier this LUN
+            relates to
+        :type npod_uuid: UUIDFilter, optional
+        :param host_uuid: Filter based on the host unique identifier on which
+            the LUN is active
+        :type host_uuid: StringFilter, optional
         :param and_filter: Concatenate another filter with a logical AND
-        :type and_filter: DataCenterFilter, optional
+        :type and_filter: LUNFilter, optional
         :param or_filter: Concatenate another filter with a logical OR
-        :type or_filter: DataCenterFilter, optional
+        :type or_filter: LUNFilter, optional
         """
 
         self.__uuid = uuid
         self.__lun_id = lun_id
+        self.__spu_serial = spu_serial
+        self.__volume_uuid = volume_uuid
+        self.__npod_uuid = npod_uuid
+        self.__host_uuid = host_uuid
         self.__and = and_filter
         self.__or = or_filter
 
     @property
-    def uuid(self) -> UuidFilter:
+    def uuid(self) -> UUIDFilter:
         """Filter based on LUN unique identifier"""
         return self.__uuid
 
@@ -95,6 +129,26 @@ class LunFilter:
         """Filter based on LUN ID"""
         return self.__lun_id
 
+    @property
+    def spu_serial(self) -> StringFilter:
+        """Filter based on the SPU serial number on which the LUN is active"""
+        return self.__spu_serial
+
+    @property
+    def volume_uuid(self) -> UUIDFilter:
+        """Filter based on the UUID of the volume this LUN relates to"""
+        return self.__volume_uuid
+
+    @property
+    def npod_uuid(self) -> UUIDFilter:
+        """Filter based on the nPod unique identifier this LUN relates to"""
+        return self.__npod_uuid
+
+    @property
+    def host_uuid(self) -> StringFilter:
+        """Filter based on the host identifier on which the LUN is active"""
+        return self.__host_uuid
+
     @property
     def and_filter(self):
         """Allows concatenation of multiple filters via logical AND"""
@@ -110,12 +164,16 @@ class LunFilter:
         result = dict()
         result["uuid"] = self.uuid
         result["lunID"] = self.lun_id
+        result["spuSerial"] = self.spu_serial
+        result["volumeUUID"] = self.volume_uuid
+        result["nPodUUID"] = self.npod_uuid
+        result["hostUUID"] = self.host_uuid
         result["and"] = self.and_filter
         result["or"] = self.or_filter
         return result
 
 
-class CreateLunInput:
+class CreateLUNInput:
     """An input object to create a LUN for a volume
 
     Allows the creation of a LUN for a volume. A LUN is an instance of a
@@ -128,6 +186,7 @@ class CreateLunInput:
             lun_id: int = None,
             host_uuids: [str] = None,
             spu_serials: [str] = None,
+            npod_lun: bool = None,
             local: bool = None
     ):
         """Constructs a new input object to create a LUN for a volume
@@ -135,32 +194,52 @@ class CreateLunInput:
         Allows the creation of a LUN for a volume. A LUN is an instance of a
         volume export that makes a volume accessible to a host.
 
-        At least one host must be specified via ``host_uuids`` or ``spu_serials`` -
-        either one option must be specified but not both. If the ``local`` option
-        is provided and set to ``True`` then the volume will be exported with
-        ALUA, otherwise with ALUA turned off.
+        Exactly one of the following properties must be specified:
+        - ``host_uuids``
+        - ``spu_serials``
+        - ``npod_lun``
+
+        If ``host_uuids`` is specified, the volume identified by ``volume_uuid``
+        will be exported to all hosts in the list of UUIDs. If ``spu_serials``
+        is provided the volume will be exported through the specified list of
+        SPU serial numbers. If ``npod_lun`` is provided, the volume will be
+        exported to all hosts in the nPod through one SPU in the host.
+
+        During export, the ``local`` option allows control over the ALUA
+        setting. If set to ``True``, the volume will be exported with ALUA
+        turned off. If set to ``False``, ALUA will be turned on for each LUN
+        that is created.
+
 
         :param volume_uuid: The unique identifier of the volume that shall be
             made available to a host
         :type volume_uuid: str
-        :param lun_id: An optional LUN ID to export volumes with a specific ID
-        :type lun_id: int
+        :param lun_id: An optional LUN ID to export volumes with a specific ID.
+            If the value is not provided, nebulon will automatically determine
+            the LUN ID based on available values.
+        :type lun_id: int, optional
         :param host_uuids: List of host UUIDs that identify the hosts the
-            volume shall be exported to. This must be provided if
-            ``spu_serials`` is not provided.
+            volume shall be exported to. Only a single LUN will be created for
+            each host in the provided list.
         :type host_uuids: [str], optional
         :param spu_serials: List of SPU serials that identify the serials the
-            volume shall be exported to. This must be provided if ``host_uuids``
-            is not provided.
+            volume shall be exported to. Only a single LUN will be created for
+            each SPU in the provided list.
         :type spu_serials: [str], optional
+        :param npod_lun: If specified and set to ``True`` a LUN will be created
+            for each host in the nPod. If not specified, this parameter defaults
+            to ``False`` and either ``host_uuids`` or ``spu_serials`` must be
+            specified.
+        :type npod_lun: bool, optional
         :param local: If provided and set to ``True`` then the volume will be
-            exported with ALUA, otherwise with ALUA turned off.
+            exported with ALUA turned off, otherwise with ALUA turned on.
         :type local: bool, optional
         """
 
         self.__volume_uuid = volume_uuid
         self.__host_uuids = host_uuids
         self.__spu_serials = spu_serials
+        self.__npod_lun = npod_lun
         self.__lun_id = lun_id
         self.__local = local
 
@@ -184,9 +263,14 @@ class CreateLunInput:
         """An optional LUN ID to assign to the volume export"""
         return self.__lun_id
 
+    @property
+    def npod_lun(self) -> bool:
+        """If ``True``, volumes will be exported to all hosts in the nPod"""
+        return self.__npod_lun
+
     @property
     def local(self) -> bool:
-        """If ``True``, volumes will be exported with ALUA turned on"""
+        """If ``True``, volumes will be exported with ALUA turned off"""
         return self.__local
 
     @property
@@ -195,12 +279,13 @@ class CreateLunInput:
         result["volumeUUID"] = self.volume_uuid
         result["hostUUIDs"] = self.host_uuids
         result["spuSerials"] = self.spu_serials
+        result["nPodLun"] = self.npod_lun
         result["lunID"] = self.lun_id
         result["local"] = self.local
         return result
 
 
-class BatchDeleteLunInput:
+class BatchDeleteLUNInput:
     """An input object to delete multiple LUNs at once"""
 
     def __init__(
@@ -214,14 +299,14 @@ class BatchDeleteLunInput:
         Either ``lun_uuids`` or ``host_uuids`` must be specified, but not both.
 
         :param volume_uuid: The unique identifier of the volume from which the
-            LUNs shall be deleted
+            LUNs shall be deleted.
         :type volume_uuid: str
         :param lun_uuids: The list of LUN identifiers that shall be deleted. If
-            ``host_uuids`` is not specified this parameter is mandatory
+            ``host_uuids`` is not specified this parameter is mandatory.
         :type lun_uuids: [str], optional
         :param host_uuids: The list of host identifiers from which the LUNs
-            shall be deleted. IF ``lun_uuids`` is not specified this parameter is
-            mandatory
+            shall be deleted. If ``lun_uuids`` is not specified this parameter
+            is mandatory.
         :type host_uuids: [str], optional
         """
 
@@ -253,8 +338,8 @@ class BatchDeleteLunInput:
         return result
 
 
-class Lun:
-    """A LUN / an export of a volume to a host
+class LUN:
+    """A LUN is an export of a volume to a host
 
     A LUN is an instance of a volume export that makes a volume accessible to
     a host.
@@ -266,7 +351,7 @@ class Lun:
     ):
         """Constructs a new LUN object
 
-        This constructor expects a dict() object from the nebulon ON API. It
+        This constructor expects a ``dict`` object from the nebulon ON API. It
         will check the returned data against the currently implemented schema
         of the SDK.
 
@@ -278,18 +363,27 @@ class Lun:
 
         self.__uuid = read_value(
             "uuid", response, str, True)
+        self.__definition_uuid = read_value(
+            "definitionUUID", response, str, True)
         self.__spu_serial = read_value(
             "spu.serial", response, str, True)
         self.__lun_id = read_value(
             "lunID", response, int, True)
         self.__volume_uuid = read_value(
-            "volume.uuid", response, str, True)
+            "volume.uuid", response, str, False)
+        self.__host_uuid = read_value(
+            "host.uuid", response, str, False)
 
     @property
     def uuid(self) -> str:
         """The unique identifier of the LUN"""
         return self.__uuid
 
+    @property
+    def definition_uuid(self) -> str:
+        """The unique identifier of the central LUN definition"""
+        return self.__definition_uuid
+
     @property
     def spu_serial(self) -> str:
         """The SPU serial where the LUN is exported from"""
@@ -305,21 +399,28 @@ class Lun:
         """The unique identifier of the volume that is exported"""
         return self.__volume_uuid
 
+    @property
+    def host_uuid(self) -> str:
+        """The unique identifier of the host the LUN is accessible by"""
+        return self.__host_uuid
+
     @staticmethod
     def fields():
         return [
             "uuid",
+            "definitionUUID",
             "spu{serial}",
+            "host{uuid}",
             "lunID",
             "volume{uuid}",
         ]
 
 
-class LunList:
+class LUNList:
     """Paginated LUN list object
 
     Contains a list of LUN objects and information for pagination. By default a
-    single page includes a maximum of `100` items unless specified otherwise
+    single page includes a maximum of ``100`` items unless specified otherwise
     in the paginated query.
 
     Consumers should always check for the property ``more`` as per default
@@ -332,7 +433,7 @@ class LunList:
     ):
         """Constructs a new LUN list object
 
-        This constructor expects a dict() object from the nebulon ON API. It
+        This constructor expects a ``dict`` object from the nebulon ON API. It
         will check the returned data against the currently implemented schema
         of the SDK.
 
@@ -348,10 +449,10 @@ class LunList:
         self.__filtered_count = read_value(
             "filteredCount", response, int, True)
         self.__items = read_value(
-            "items", response, Lun, True)
+            "items", response, LUN, True)
 
     @property
-    def items(self) -> [Lun]:
+    def items(self) -> [LUN]:
         """List of LUN objects in the pagination list"""
         return self.__items
 
@@ -373,38 +474,38 @@ class LunList:
     @staticmethod
     def fields():
         return [
-            "items{%s}" % (",".join(Lun.fields())),
+            "items{%s}" % (",".join(LUN.fields())),
             "more",
             "totalCount",
             "filteredCount",
         ]
 
 
-class LunsMixin(NebMixin):
+class LUNsMixin(NebMixin):
     """Mixin to add LUN related methods to the GraphQL client"""
 
     def get_luns(
             self,
             page: PageInput = None,
-            lun_filter: LunFilter = None,
-            sort: LunSort = None
-    ) -> LunList:
+            lun_filter: LUNFilter = None,
+            sort: LUNSort = None
+    ) -> LUNList:
         """Retrieves a list of LUN objects
 
         :param page: The requested page from the server. This is an optional
             argument and if omitted the server will default to returning the
-            first page with a maximum of `100` items.
+            first page with a maximum of ``100`` items.
         :type page: PageInput, optional
         :param lun_filter: A filter object to filter the LUNs on the
             server. If omitted, the server will return all objects as a
             paginated response.
-        :type lun_filter: LunFilter, optional
+        :type lun_filter: LUNFilter, optional
         :param sort: A sort definition object to sort the LUN objects
             on supported properties. If omitted objects are returned in the
             order as they were created in.
-        :type sort: LunSort, optional
+        :type sort: LUNSort, optional
 
-        :returns LunList: A paginated list of LUNs.
+        :returns LUNList: A paginated list of LUNs.
 
         :raises GraphQLError: An error with the GraphQL endpoint.
         """
@@ -422,46 +523,25 @@ class LunsMixin(NebMixin):
         response = self._query(
             name="getLUNs",
             params=parameters,
-            fields=LunList.fields()
+            fields=LUNList.fields()
         )
 
         # convert to object
-        return LunList(response)
+        return LUNList(response)
 
     def create_lun(
             self,
-            volume_uuid: str,
-            lun_id: int = None,
-            host_uuids: [str] = None,
-            spu_serials: [str] = None,
-            local: bool = None
-    ):
+            lun_input: CreateLUNInput
+    ) -> LUN:
         """Allows creation of a new LUN
 
         Allows the creation of a LUN for a volume. A LUN is an instance of a
         volume export that makes a volume accessible to a host.
 
-        At least one host must be specified via ``host_uuids`` or ``spu_serials`` -
-        either one option must be specified but not both. If the ``local`` option
-        is provided and set to ``True`` then the volume will be exported with
-        ALUA, otherwise with ALUA turned off.
+        :param lun_input: The parameters that describe the LUN or LUNs to create
+        :type lun_input: CreateLUNInput
 
-        :param volume_uuid: The unique identifier of the volume that shall be
-            made available to a host
-        :type volume_uuid: str
-        :param lun_id: An optional LUN ID to export volumes with a specific ID
-        :type lun_id: int
-        :param host_uuids: List of host UUIDs that identify the hosts the
-            volume shall be exported to. This must be provided if
-            ``spu_serials`` is not provided.
-        :type host_uuids: [str], optional
-        :param spu_serials: List of SPU serials that identify the serials the
-            volume shall be exported to. This must be provided if ``host_uuids``
-            is not provided.
-        :type spu_serials: [str], optional
-        :param local: If provided and set to ``True`` then the volume will be
-            exported with ALUA, otherwise with ALUA turned off.
-        :type local: bool, optional
+        :returns LUN: the created LUN.
 
         :raises GraphQLError: An error with the GraphQL endpoint.
         """
@@ -469,27 +549,85 @@ class LunsMixin(NebMixin):
         # setup query parameters
         parameters = dict()
         parameters["input"] = GraphQLParam(
-            CreateLunInput(
-                volume_uuid=volume_uuid,
-                host_uuids=host_uuids,
-                spu_serials=spu_serials,
-                lun_id=lun_id,
-                local=local
-            ),
+            lun_input,
             "CreateLUNInput",
             True
         )
 
         # make the request
         response = self._mutation(
-            name="createLUN",
+            name="createLUNV2",
             params=parameters,
             fields=TokenResponse.fields()
         )
 
         # convert to object and deliver token
         token_response = TokenResponse(response)
-        token_response.deliver_token()
+        delivery_response = token_response.deliver_token()
+
+        # wait for recipe completion
+        # TODO: Nebulon ON now returns a different response
+        recipe_uuid = delivery_response["recipe_uuid_to_wait_on"]
+        npod_uuid = delivery_response["npod_uuid_to_wait_on"]
+        npod_recipe_filter = NPodRecipeFilter(
+                npod_uuid=npod_uuid,
+                recipe_uuid=recipe_uuid)
+
+        # set a custom timeout for the create lun process
+        start = datetime.now()
+
+        while True:
+            sleep(5)
+
+            recipes = self.get_npod_recipes(npod_recipe_filter=npod_recipe_filter)
+
+            # if there is no record in the cloud wait a few more seconds
+            # this case should not exist, but is a safety measure for a
+            # potential race condition
+            if len(recipes.items) != 0:
+
+                # based on the query there should be exactly one
+                recipe = recipes.items[0]
+
+                if recipe.state == RecipeState.Failed:
+                    raise Exception(f"create lun failed: {recipe.status}")
+
+                if recipe.state == RecipeState.Timeout:
+                    raise Exception(f"create lun timeout: {recipe.status}")
+
+                if recipe.state == RecipeState.Cancelled:
+                    raise Exception(f"create lun cancelled: {recipe.status}")
+
+                if recipe.state == RecipeState.Completed:
+                    lun_list = self.get_luns(
+                        lun_filter=LUNFilter(
+                            npod_uuid=UUIDFilter(
+                                equals=npod_uuid
+                            ),
+                            and_filter=LUNFilter(
+                                volume_uuid=UUIDFilter(
+                                    equals=lun_input.volume_uuid
+                                ),
+                                and_filter=LUNFilter(
+                                    lun_id=IntFilter(
+                                        equals=lun_input.lun_id
+                                    )
+                                )
+                            )
+                        )
+                    )
+
+                    if lun_list.filtered_count == 0:
+                        break
+
+                    return lun_list.items[0]
+
+            # Wait time remaining until timeout
+            total_duration = (datetime.now() - start).total_seconds()
+            time_remaining = _TIMEOUT_SECONDS - total_duration
+
+            if time_remaining <= 0:
+                raise Exception("create lun timed out")
 
     def delete_lun(
             self,
@@ -520,22 +658,14 @@ class LunsMixin(NebMixin):
 
     def delete_luns(
             self,
-            volume_uuid: str,
-            lun_uuids: [str] = None,
-            host_uuids: [str] = None
+            batch_delete_lun_input: BatchDeleteLUNInput
     ):
-        """Allows deletion of multiple LUNs
+        """Allows deletion of multiple LUNs simultaneously
+
+        :param batch_delete_lun_input: An input parameter describing the
+            selection criteria for the LUNs to delete.
+        :type batch_delete_lun_input: BatchDeleteLUNInput
 
-        :param volume_uuid: The unique identifier of the volume from which the
-            LUNs shall be deleted
-        :type volume_uuid: str
-        :param lun_uuids: The list of LUN identifiers that shall be deleted. If
-            ``host_uuids`` is not specified this parameter is mandatory
-        :type lun_uuids: [str], optional
-        :param host_uuids: The list of host identifiers from which the LUNs
-            shall be deleted. IF ``lun_uuids`` is not specified this parameter is
-            mandatory
-        :type host_uuids: [str], optional
 
         :raises GraphQLError: An error with the GraphQL endpoint.
         """
@@ -543,11 +673,7 @@ class LunsMixin(NebMixin):
         # setup query parameters
         parameters = dict()
         parameters["input"] = GraphQLParam(
-            BatchDeleteLunInput(
-                volume_uuid=volume_uuid,
-                lun_uuids=lun_uuids,
-                host_uuids=host_uuids
-            ),
+            batch_delete_lun_input,
             "BatchDeleteLUNInput",
             True
         )
