@@ -1,5 +1,5 @@
 #
-# Copyright 2020 Nebulon, Inc.
+# Copyright 2021 Nebulon, Inc.
 # All Rights Reserved.
 #
 # DISCLAIMER: THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND,
@@ -13,7 +13,7 @@
 
 from .graphqlclient import GraphQLParam, NebMixin
 from .common import PageInput, read_value
-from .filters import StringFilter, UuidFilter
+from .filters import StringFilter, UUIDFilter
 from .sorting import SortDirection
 
 __all__ = [
@@ -38,11 +38,12 @@ class NPodTemplateFilter:
 
     def __init__(
             self,
-            uuid: UuidFilter = None,
+            uuid: UUIDFilter = None,
             name: StringFilter = None,
             os: StringFilter = None,
             app: StringFilter = None,
             nebulon_template: bool = None,
+            only_last_version: bool = None,
             and_filter=None,
             or_filter=None
     ):
@@ -60,6 +61,9 @@ class NPodTemplateFilter:
             provided templates. If set to ``False`` only custom templates
             will be returned.
         :type nebulon_template: bool, optional
+        :param only_last_version: If set to ``True`` only the latest version of
+            each nPod template will be returned
+        :type only_last_version: bool, optional
         :param and_filter: Concatenate another filter with a logical AND
         :type and_filter: NPodTemplateFilter, optional
         :param or_filter: Concatenate another filter with a logical OR
@@ -71,11 +75,12 @@ class NPodTemplateFilter:
         self.__os = os
         self.__app = app
         self.__nebulon_template = nebulon_template
+        self.__only_last_version = only_last_version
         self.__and = and_filter
         self.__or = or_filter
 
     @property
-    def uuid(self) -> UuidFilter:
+    def uuid(self) -> UUIDFilter:
         """Filter based on nPod template unique identifier"""
         return self.__uuid
 
@@ -99,6 +104,11 @@ class NPodTemplateFilter:
         """Filter nPod templates for nebulon templates or custom templates"""
         return self.__nebulon_template
 
+    @property
+    def only_last_version(self) -> bool:
+        """Filter nPod templates for only their latest version"""
+        return self.__only_last_version
+
     @property
     def and_filter(self):
         """Allows concatenation of multiple filters via logical AND"""
@@ -117,6 +127,7 @@ class NPodTemplateFilter:
         result["os"] = self.os
         result["app"] = self.app
         result["nebulonTemplate"] = self.nebulon_template
+        result["onlyLastVersion"] = self.only_last_version
         result["and"] = self.and_filter
         result["or"] = self.or_filter
         return result
@@ -145,18 +156,6 @@ class NPodTemplateSort:
         :type app: SortDirection, optional
         """
 
-        # check the basics
-        param_count = 0
-        if name is not None:
-            param_count += 1
-        if os is not None:
-            param_count += 1
-        if app is not None:
-            param_count += 1
-
-        if param_count != 1:
-            raise ValueError("Exactly one parameter must be specified")
-
         self.__name = name
         self.__os = os
         self.__app = app
@@ -202,7 +201,7 @@ class CreateNPodTemplateInput:
             boot_volume: bool,
             os: str,
             volume_size_bytes: int = None,
-            shared_volume: bool = None,
+            shared_lun: bool = None,
             boot_volume_size_bytes: int = None,
             boot_image_url: str = None,
             app: str = None,
@@ -218,7 +217,7 @@ class CreateNPodTemplateInput:
         :type name: str
         :param saving_factor: The anticipated saving factor for the specified
             application after data compression and data deduplication. Allowed
-            values are between `1.0` and `10.0`. nebulon ON will use this
+            values are between ``1.0`` and ``10.0``. nebulon ON will use this
             assumption for provisioning storage volumes.
         :type saving_factor: float
         :param mirrored_volume: Specifies if volumes shall be mirrored for
@@ -234,18 +233,18 @@ class CreateNPodTemplateInput:
         :type os: str
         :param volume_size_bytes: The size of volumes to create in bytes. Either
             volume size or volume count must be present.
-        :type volume_size_bytes: int
-        :param shared_volume: Allows configuring volume export options. If set
+        :type volume_size_bytes: int, optional
+        :param shared_lun: Allows configuring volume export options. If set
             to ``True`` all volumes except boot volumes will be made available
             to each host / server in the nPod for read and write access. If set
             to ``False`` volumes will only be made available to the local host
             of every SPU. By default volumes are created as shared volumes.
-        :type shared_volume: bool, optional
+        :type shared_lun: bool, optional
         :param boot_volume_size_bytes: The size of the boot volume to create
             in bytes. This value is only considered when the parameter
             ``boot_volume`` is set to ``True``.
         :type boot_volume_size_bytes: int, optional
-        :param boot_image_url: Allows specifying an HTTP(s) URL for a boot
+        :param boot_image_url: Allows specifying an HTTP(S) URL for a boot
             image that is applied to the boot volume when an nPod is created.
         :type boot_image_url: str, optional
         :param app: The name of the application that will be running on the
@@ -269,7 +268,7 @@ class CreateNPodTemplateInput:
         self.__volume_size_bytes = volume_size_bytes
         self.__saving_factor = saving_factor
         self.__mirrored_volume = mirrored_volume
-        self.__shared_volume = shared_volume
+        self.__shared_lun = shared_lun
         self.__boot_volume = boot_volume
         self.__boot_volume_size_bytes = boot_volume_size_bytes
         self.__boot_image_url = boot_image_url
@@ -301,13 +300,13 @@ class CreateNPodTemplateInput:
         return self.__mirrored_volume
 
     @property
-    def shared_volume(self) -> bool:
+    def shared_lun(self) -> bool:
         """Indicates if volumes are shared between all hosts in an nPod"""
-        return self.__shared_volume
+        return self.__shared_lun
 
     @property
     def boot_volume(self) -> bool:
-        """Indicates if a boot volume for the O/S will be provisioned"""
+        """Indicates if a boot volume for the OS will be provisioned"""
         return self.__boot_volume
 
     @property
@@ -317,7 +316,7 @@ class CreateNPodTemplateInput:
 
     @property
     def boot_image_url(self) -> str:
-        """Allows specifying a URL to an O/S image for the boot volume"""
+        """Allows specifying a URL to an OS image for the boot volume"""
         return self.__boot_image_url
 
     @property
@@ -352,7 +351,7 @@ class CreateNPodTemplateInput:
         result["volumeSizeBytes"] = self.volume_size_bytes
         result["savingFactor"] = self.saving_factor
         result["mirroredVolume"] = self.mirrored_volume
-        result["sharedLUN"] = self.shared_volume
+        result["sharedLUN"] = self.shared_lun
         result["bootVolume"] = self.boot_volume
         result["bootVolumeSizeBytes"] = self.boot_volume_size_bytes
         result["bootImageURL"] = self.boot_image_url
@@ -379,7 +378,7 @@ class UpdateNPodTemplateInput:
             volume_size_bytes: int = None,
             saving_factor: float = None,
             mirrored_volume: bool = None,
-            shared_volume: bool = None,
+            shared_lun: bool = None,
             boot_volume: bool = None,
             boot_volume_size_bytes: int = None,
             boot_image_url: str = None,
@@ -397,7 +396,7 @@ class UpdateNPodTemplateInput:
         :type name: str
         :param saving_factor: The anticipated saving factor for the specified
             application after data compression and data deduplication. Allowed
-            values are between `1.0` and `10.0`. nebulon ON will use this
+            values are between ``1.0`` and ``10.0``. nebulon ON will use this
             assumption for provisioning storage volumes.
         :type saving_factor: float, optional
         :param mirrored_volume: Specifies if volumes shall be mirrored for
@@ -414,12 +413,12 @@ class UpdateNPodTemplateInput:
         :param volume_size_bytes: The size of volumes to create in bytes. Either
             volume size or volume count must be present.
         :type volume_size_bytes: int, optional
-        :param shared_volume: Allows configuring volume export options. If set
+        :param shared_lun: Allows configuring volume export options. If set
             to ``True`` all volumes except boot volumes will be made available
             to each host / server in the nPod for read and write access. If set
             to ``False`` volumes will only be made available to the local host
             of every SPU. By default volumes are created as shared volumes.
-        :type shared_volume: bool, optional
+        :type shared_lun: bool, optional
         :param boot_volume_size_bytes: The size of the boot volume to create
             in bytes. This value is only considered when the parameter
             ``boot_volume`` is set to ``True``.
@@ -448,7 +447,7 @@ class UpdateNPodTemplateInput:
         self.__volume_size_bytes = volume_size_bytes
         self.__saving_factor = saving_factor
         self.__mirrored_volume = mirrored_volume
-        self.__shared_volume = shared_volume
+        self.__shared_lun = shared_lun
         self.__boot_volume = boot_volume
         self.__boot_volume_size_bytes = boot_volume_size_bytes
         self.__boot_image_url = boot_image_url
@@ -480,13 +479,13 @@ class UpdateNPodTemplateInput:
         return self.__mirrored_volume
 
     @property
-    def shared_volume(self) -> bool:
+    def shared_lun(self) -> bool:
         """Indicates if volumes are shared between all hosts in an nPod"""
-        return self.__shared_volume
+        return self.__shared_lun
 
     @property
     def boot_volume(self) -> bool:
-        """Indicates if a boot volume for the O/S will be provisioned"""
+        """Indicates if a boot volume for the OS will be provisioned"""
         return self.__boot_volume
 
     @property
@@ -496,7 +495,7 @@ class UpdateNPodTemplateInput:
 
     @property
     def boot_image_url(self) -> str:
-        """Allows specifying a URL to an O/S image for the boot volume"""
+        """Allows specifying a URL to an OS image for the boot volume"""
         return self.__boot_image_url
 
     @property
@@ -531,7 +530,7 @@ class UpdateNPodTemplateInput:
         result["volumeSizeBytes"] = self.volume_size_bytes
         result["savingFactor"] = self.saving_factor
         result["mirroredVolume"] = self.mirrored_volume
-        result["sharedVolume"] = self.shared_volume
+        result["sharedVolume"] = self.shared_lun
         result["bootVolume"] = self.boot_volume
         result["bootVolumeSizeBytes"] = self.boot_volume_size_bytes
         result["bootImageURL"] = self.boot_image_url
@@ -559,7 +558,7 @@ class NPodTemplate:
     ):
         """Constructs a new nPod template object
 
-        This constructor expects a dict() object from the nebulon ON API. It
+        This constructor expects a ``dict`` object from the nebulon ON API. It
         will check the returned data against the currently implemented schema
         of the SDK.
 
@@ -610,7 +609,7 @@ class NPodTemplate:
 
     @property
     def uuid(self) -> str:
-        """The unique identifier of the template if versioned"""
+        """The unique identifier of the template version"""
         return self.__uuid
 
     @property
@@ -640,7 +639,7 @@ class NPodTemplate:
 
     @property
     def boot_volume(self) -> bool:
-        """Indicates if a boot volume for the O/S will be provisioned"""
+        """Indicates if a boot volume for the OS will be provisioned"""
         return self.__boot_volume
 
     @property
@@ -650,7 +649,7 @@ class NPodTemplate:
 
     @property
     def boot_image_url(self) -> str:
-        """Allows specifying a URL to an O/S image for the boot volume"""
+        """Allows specifying a URL to an OS image for the boot volume"""
         return self.__boot_image_url
 
     @property
@@ -715,7 +714,7 @@ class NPodTemplateList:
     """Paginated nPod template list object
 
     Contains a list of nPod template objects and information for
-    pagination. By default a single page includes a maximum of `100` items
+    pagination. By default a single page includes a maximum of ``100`` items
     unless specified otherwise in the paginated query.
 
     Consumers should always check for the property ``more`` as per default
@@ -728,7 +727,7 @@ class NPodTemplateList:
     ):
         """Constructs a new nPod template list object
 
-        This constructor expects a dict() object from the nebulon ON API. It
+        This constructor expects a ``dict`` object from the nebulon ON API. It
         will check the returned data against the currently implemented schema
         of the SDK.
 
@@ -790,7 +789,7 @@ class NPodTemplateMixin(NebMixin):
 
         :param page: The requested page from the server. This is an optional
             argument and if omitted the server will default to returning the
-            first page with a maximum of `100` items.
+            first page with a maximum of ``100`` items.
         :type page: PageInput, optional
         :param template_filter: A filter object to filter the nPod templates
             on the server. If omitted, the server will return all objects as a
@@ -827,73 +826,13 @@ class NPodTemplateMixin(NebMixin):
 
     def create_npod_template(
             self,
-            name: str,
-            saving_factor: float,
-            mirrored_volume: bool,
-            boot_volume: bool,
-            os: str,
-            volume_size_bytes: int = None,
-            shared_volume: bool = None,
-            boot_volume_size_bytes: int = None,
-            boot_image_url: str = None,
-            app: str = None,
-            note: str = None,
-            snapshot_schedule_template_uuids: [str] = None,
-            volume_count: int = None
+            create_npod_template_input: CreateNPodTemplateInput
     ) -> NPodTemplate:
         """Create a new nPod template
 
-        :param name: The name of the nPod template to update. The name cannot
-            be changed. If users want to change the name of a nPod template they
-            should clone the template with a new name and delete the old record
-        :type name: str
-        :param saving_factor: The anticipated saving factor for the specified
-            application after data compression and data deduplication. Allowed
-            values are between `1.0` and `10.0`. nebulon ON will use this
-            assumption for provisioning storage volumes.
-        :type saving_factor: float
-        :param mirrored_volume: Specifies if volumes shall be mirrored for
-            high availability. If set to ``True`` two copies of the same volume
-            will be created in an nPod on different SPUs for high availability.
-        :type mirrored_volume: bool
-        :param boot_volume: If set to ``True`` nebulon ON will provision a
-            boot volume for the server's operating system. If set, the parameter
-            ``boot_volume_size_bytes`` must also be specified.
-        :type boot_volume: bool
-        :param os: The name of the operating system that will be installed on
-            servers in the nPod.
-        :type os: str
-        :param volume_size_bytes: The size of volumes to create in bytes. Either
-            volume size or volume count must be present.
-        :type volume_size_bytes: int
-        :param shared_volume: Allows configuring volume export options. If set
-            to ``True`` all volumes except boot volumes will be made available
-            to each host / server in the nPod for read and write access. If set
-            to ``False`` volumes will only be made available to the local host
-            of every SPU. By default volumes are created as shared volumes.
-        :type shared_volume: bool, optional
-        :param boot_volume_size_bytes: The size of the boot volume to create
-            in bytes. This value is only considered when the parameter
-            ``boot_volume`` is set to ``True``.
-        :type boot_volume_size_bytes: int, optional
-        :param boot_image_url: Allows specifying an HTTP(s) URL for a boot
-            image that is applied to the boot volume when an nPod is created.
-        :type boot_image_url: str, optional
-        :param app: The name of the application that will be running on the
-            nPod.
-        :type app: str, optional
-        :param note: An optional note for the nPod template
-        :type note: str, optional
-        :param snapshot_schedule_template_uuids: Allows specifying snapshot
-            schedule templates that will be automatically created for any
-            derived nPods after nPod creation.
-        :type snapshot_schedule_template_uuids: [str], optional
-        :param volume_count: Allows specifying a volume count. This option is
-            only allowed when ``shared_volume`` is set to ``False`` and allows
-            creating a specific number of volumes per host / server. This is
-            useful when the size of the volume does not matter but the number
-            of volumes is important
-        :type volume_count: int, optional
+        :param create_npod_template_input: A parameter that describes the nPod
+            template properties to create
+        :type create_npod_template_input: CreateNPodTemplateInput
 
         :returns NPodTemplate: The created nPod template
 
@@ -903,22 +842,7 @@ class NPodTemplateMixin(NebMixin):
         # setup query parameters
         parameters = dict()
         parameters["input"] = GraphQLParam(
-            CreateNPodTemplateInput(
-                name=name,
-                saving_factor=saving_factor,
-                mirrored_volume=mirrored_volume,
-                boot_volume=boot_volume,
-                os=os,
-                volume_size_bytes=volume_size_bytes,
-                shared_volume=shared_volume,
-                boot_volume_size_bytes=boot_volume_size_bytes,
-                boot_image_url=boot_image_url,
-                app=app,
-                note=note,
-                snapshot_schedule_template_uuids=
-                snapshot_schedule_template_uuids,
-                volume_count=volume_count
-            ),
+            create_npod_template_input,
             "CreateNPodTemplateInput",
             True
         )
@@ -935,19 +859,7 @@ class NPodTemplateMixin(NebMixin):
 
     def update_npod_template(
             self,
-            name: str,
-            volume_size_bytes: int = None,
-            saving_factor: float = None,
-            mirrored_volume: bool = None,
-            shared_volume: bool = None,
-            boot_volume: bool = None,
-            boot_volume_size_bytes: int = None,
-            boot_image_url: str = None,
-            os: str = None,
-            app: str = None,
-            note: str = None,
-            snapshot_schedule_template_uuids: [str] = None,
-            volume_count: int = None
+            update_npod_template_input: UpdateNPodTemplateInput
     ) -> NPodTemplate:
         """Update nPod template properties
 
@@ -956,57 +868,11 @@ class NPodTemplateMixin(NebMixin):
         original nPod template is accessible via the nPod template
         ``parent_uuid`` property.
 
-        :param name: The name of the nPod template to update. The name cannot
-            be changed. If users want to change the name of a nPod template they
-            should clone the template with a new name and delete the old record
-        :type name: str
-        :param saving_factor: The anticipated saving factor for the specified
-            application after data compression and data deduplication. Allowed
-            values are between `1.0` and `10.0`. nebulon ON will use this
-            assumption for provisioning storage volumes.
-        :type saving_factor: float, optional
-        :param mirrored_volume: Specifies if volumes shall be mirrored for
-            high availability. If set to ``True`` two copies of the same volume
-            will be created in an nPod on different SPUs for high availability.
-        :type mirrored_volume: bool, optional
-        :param boot_volume: If set to ``True`` nebulon ON will provision a
-            boot volume for the server's operating system. If set, the parameter
-            ``boot_volume_size_bytes`` must also be specified.
-        :type boot_volume: bool, optional
-        :param os: The name of the operating system that will be installed on
-            servers in the nPod.
-        :type os: str, optional
-        :param volume_size_bytes: The size of volumes to create in bytes. Either
-            volume size or volume count must be present.
-        :type volume_size_bytes: int, optional
-        :param shared_volume: Allows configuring volume export options. If set
-            to ``True`` all volumes except boot volumes will be made available
-            to each host / server in the nPod for read and write access. If set
-            to ``False`` volumes will only be made available to the local host
-            of every SPU. By default volumes are created as shared volumes.
-        :type shared_volume: bool, optional
-        :param boot_volume_size_bytes: The size of the boot volume to create
-            in bytes. This value is only considered when the parameter
-            ``boot_volume`` is set to ``True``.
-        :type boot_volume_size_bytes: int, optional
-        :param boot_image_url: Allows specifying an HTTP(s) URL for a boot
-            image that is applied to the boot volume when an nPod is created.
-        :type boot_image_url: str, optional
-        :param app: The name of the application that will be running on the
-            nPod.
-        :type app: str, optional
-        :param note: An optional note for the nPod template
-        :type note: str, optional
-        :param snapshot_schedule_template_uuids: Allows specifying snapshot
-            schedule templates that will be automatically created for any
-            derived nPods after nPod creation.
-        :type snapshot_schedule_template_uuids: [str], optional
-        :param volume_count: Allows specifying a volume count. This option is
-            only allowed when ``shared_volume`` is set to ``False`` and allows
-            creating a specific number of volumes per host / server. This is
-            useful when the size of the volume does not matter but the number
-            of volumes is important
-        :type volume_count: int, optional
+        :param update_npod_template_input: A parameter that describes all
+            modifications to make on a selected nPod template. The target nPod
+            template is identified by the ``name`` property of the supplied
+            ``UpdateNPodTemplateInput`` object.
+        :type update_npod_template_input: UpdateNPodTemplateInput
 
         :returns NPodTemplate: The updated nPod template
 
@@ -1016,22 +882,7 @@ class NPodTemplateMixin(NebMixin):
         # setup query parameters
         parameters = dict()
         parameters["input"] = GraphQLParam(
-            UpdateNPodTemplateInput(
-                name=name,
-                saving_factor=saving_factor,
-                mirrored_volume=mirrored_volume,
-                boot_volume=boot_volume,
-                os=os,
-                volume_size_bytes=volume_size_bytes,
-                shared_volume=shared_volume,
-                boot_volume_size_bytes=boot_volume_size_bytes,
-                boot_image_url=boot_image_url,
-                app=app,
-                note=note,
-                snapshot_schedule_template_uuids=
-                snapshot_schedule_template_uuids,
-                volume_count=volume_count
-            ),
+            update_npod_template_input,
             "UpdateNPodTemplateInput",
             True
         )
@@ -1063,13 +914,8 @@ class NPodTemplateMixin(NebMixin):
         :returns bool: If the deletion was successful
 
         :raises GraphQLError: An error with the GraphQL endpoint.
-        :raises ValueError: When illegal parameters are supplied.
         """
 
-        # check the basics
-        if parent_uuid is None or len(parent_uuid) == 0:
-            raise ValueError("parent_uuid must be a valid UUID string")
-
         # setup query parameters
         parameters = dict()
         parameters["parentUUID"] = GraphQLParam(
