@@ -1,5 +1,5 @@
 #
-# Copyright 2020 Nebulon, Inc.
+# Copyright 2021 Nebulon, Inc.
 # All Rights Reserved.
 #
 # DISCLAIMER: THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND,
@@ -13,19 +13,24 @@
 
 from .graphqlclient import GraphQLParam, NebMixin
 from datetime import datetime
-from .common import NebEnum, read_value
+from .common import NebEnum, read_value, PageInput
 from .issues import Issues
 from .tokens import TokenResponse
+from .sorting import SortDirection
+from .filters import StringFilter
 
 __all__ = [
     "NebPackagePriority",
     "NebPackageType",
     "PackageInfo",
     "RecommendedPackages",
-    "UpdatePackages",
+    "AvailablePackagesSort",
+    "AvailablePackagesFilter",
+    "PackageInfoList",
     "UpdateStateSpu",
     "UpdateHistory",
-    "UpdatesMixin"
+    "UpdatesMixin",
+    "NPodRecommendedPackage",
 ]
 
 
@@ -33,7 +38,7 @@ class NebPackagePriority(NebEnum):
     """Indicates the importance for installing a nebulon package"""
 
     Normal = "Normal"
-    """Indicates a routing installation package"""
+    """Indicates a routine installation package"""
 
     Critical = "Critical"
     """Indicates a critical update"""
@@ -48,16 +53,144 @@ class NebPackageType(NebEnum):
     Patch = "Patch"
     """A patch package that resolves a specific issue with nebOS"""
 
-    Recovery = "Recovery"
-    """A package that is used for recovery purposes"""
+
+class PackageSupportState(NebEnum):
+    """Indicates the support state for a nebOS software package"""
+
+    Supported = "Supported"
+    """A supported software package"""
+
+    EndOfSupport = "EndOfSupport"
+    """A software package that is no longer receives active support"""
+
+
+class AvailablePackagesSort:
+    """A sort object for services processing units (SPU)
+
+    Allows sorting SPUs on common properties. The sort object allows only one
+    property to be specified.
+    """
+
+    def __init__(
+            self,
+            package_name: SortDirection = None,
+            release_date: SortDirection = None,
+    ):
+        """Constructs a new sort object for SPUs
+
+        Allows sorting software packages on common properties. The sort object
+        allows only one property to be specified.
+
+        :param package_name: Sort direction for the ``package_name`` property
+        :type package_name: SortDirection, optional
+        :param release_date: Sort direction for the ``release_date`` property
+        :type release_date: SortDirection, optional
+        """
+
+        self.__package_name = package_name
+        self.__release_date = release_date
+
+    @property
+    def package_name(self) -> SortDirection:
+        """Sort direction for the ``serial`` property"""
+        return self.__package_name
+
+    @property
+    def release_date(self) -> SortDirection:
+        """Sort direction for the ``serial`` property"""
+        return self.__release_date
+
+    @property
+    def as_dict(self):
+        result = dict()
+        result["packageName"] = self.package_name
+        result["releaseDate"] = self.release_date
+        return result
+
+
+class AvailablePackagesFilter:
+    """A filter object to filter software packages
+
+    Allows filtering for specific software packages in nebulon ON. The
+    filter allows only one property to be specified. If filtering on multiple
+    properties is needed, use the ``and_filter`` and ``or_filter`` options to
+    concatenate multiple filters.
+    """
+
+    def __init__(
+            self,
+            package_name: StringFilter = None,
+            package_type: NebPackageType = None,
+            package_priority: NebPackagePriority = None,
+            and_filter=None,
+            or_filter=None
+    ):
+        """Constructs a new filter object
+
+        The filter allows only one property to be specified. If filtering on
+        multiple properties is needed, use the ``and_filter`` and ``or_filter``
+        options to concatenate multiple filters.
+
+        :param package_name: Filter based on package name
+        :type package_name: StringFilter, optional
+        :param package_type: Filter by package type
+        :type package_type: NebPackageType, optional
+        :param package_priority: Filter by package priority
+        :type package_priority: NebPackagePriority, optional
+        :param and_filter: Concatenate another filter with a logical AND
+        :type and_filter: SpuFilter, optional
+        :param or_filter: Concatenate another filter with a logical OR
+        :type or_filter: SpuFilter, optional
+        """
+
+        self.__package_name = package_name
+        self.__package_type = package_type
+        self.__package_priority = package_priority
+        self.__and = and_filter
+        self.__or = or_filter
+
+    @property
+    def package_name(self) -> StringFilter:
+        """Filter based on package name"""
+        return self.__package_name
+
+    @property
+    def package_type(self) -> NebPackageType:
+        """Filter by package type"""
+        return self.__package_type
+
+    @property
+    def package_priority(self) -> NebPackagePriority:
+        """Filter by package priority"""
+        return self.__package_priority
+
+    @property
+    def and_filter(self):
+        """Allows concatenation of multiple filters via logical AND"""
+        return self.__and
+
+    @property
+    def or_filter(self):
+        """Allows concatenation of multiple filters via logical OR"""
+        return self.__or
+
+    @property
+    def as_dict(self):
+        result = dict()
+        result["packageName"] = self.package_name
+        result["packageType"] = self.package_type
+        result["packagePriority"] = self.package_priority
+        result["and"] = self.and_filter
+        result["or"] = self.or_filter
+        return result
 
 
 class PackageInfo:
     """A nebulon update package
 
-    Allows updating nebulon services processing units to a specific nebOS
-    version. The package information object describes each released nebOS
-    package.
+    Represents a software bundle for nebulon services processing units
+    and all related metadata. This information is used for updating
+    nebulon services processing units to a specific nebOS version.
     """
 
     def __init__(
@@ -66,7 +199,7 @@ class PackageInfo:
     ):
         """Constructs a new package information object
 
-        This constructor expects a dict() object from the nebulon ON API. It
+        This constructor expects a ``dict`` object from the nebulon ON API. It
         will check the returned data against the currently implemented schema
         of the SDK.
 
@@ -78,7 +211,7 @@ class PackageInfo:
         self.__package_name = read_value(
             "packageName", response, str, True)
         self.__package_size_bytes = read_value(
-            "packageSize", response, int, True)
+            "packageSizeBytes", response, int, True)
         self.__release_notes_url = read_value(
             "releaseNotesURL", response, str, True)
         self.__prerequisites = read_value(
@@ -89,16 +222,20 @@ class PackageInfo:
             "packageType", response, NebPackageType, True)
         self.__package_priority = read_value(
             "packagePriority", response, NebPackagePriority, True)
-        self.__package_deprecated = read_value(
-            "packageDeprecated", response, bool, True)
-        self.__release_unix = read_value(
-            "releaseUnix", response, int, True)
-        self.__eligible_npod_uuids = read_value(
-            "eligiblePods.uid", response, str, False)
+        self.__release_date = read_value(
+            "releaseDate", response, datetime, True)
         self.__version_number = read_value(
             "versionNumber", response, str, True)
         self.__patch_number = read_value(
             "patchNumber", response, str, False)
+        self.__support_state = read_value(
+            "supportState", response, PackageSupportState, True)
+        self.__lts_version = read_value(
+            "longTermSupportVersion", response, bool, True)
+        self.__offline = read_value(
+            "offlineCheck", response, str, True)
+        self.__eligible_npod_uuids = read_value(
+            "eligibleNPods.uuid", response, str, False)
 
     @property
     def package_name(self) -> str:
@@ -136,17 +273,17 @@ class PackageInfo:
         return self.__package_priority
 
     @property
-    def package_deprecated(self) -> bool:
-        """Indicates it the package is deprecated and no longer available"""
-        return self.__package_deprecated
+    def support_state(self) -> PackageSupportState:
+        """Indicates the package's support state"""
+        return self.__support_state
 
     @property
-    def release_unix(self) -> int:
-        """The release date as a UNIX timestamp"""
-        return self.__release_unix
+    def release_date(self) -> datetime:
+        """The release date"""
+        return self.__release_date
 
     @property
-    def eligible_npod_uuids(self) -> list:
+    def eligible_npod_uuids(self) -> [str]:
         """List of nPod unique identifiers that can install this package"""
         return self.__eligible_npod_uuids
 
@@ -160,24 +297,90 @@ class PackageInfo:
         """The patch number if it is a patch"""
         return self.__patch_number
 
+    @property
+    def lts_version(self) -> bool:
+        """Indicates if the software package is under long-term support"""
+        return self.__lts_version
+
+    @property
+    def offline(self) -> bool:
+        """Indicates if the update is done offline"""
+        return self.__offline
+
     @staticmethod
     def fields():
         return [
             "packageName",
-            "packageSize",
+            "packageSizeBytes",
             "releaseNotesURL",
             "prerequisites",
             "packageDescription",
             "packageType",
             "packagePriority",
-            "packageDeprecated",
-            "releaseUnix",
-            "eligiblePods{uid}",
+            "releaseDate",
+            "supportState",
+            "longTermSupportVersion",
+            "eligibleNPods{uuid}",
+            "offlineCheck",
             "versionNumber",
             "patchNumber",
         ]
 
 
+class NPodRecommendedPackage:
+    """A nebulon update recommendation
+
+    This information is used to indicate package installation recommendations
+    for nPods.
+    """
+
+    def __init__(
+            self,
+            response: dict
+    ):
+        """Constructs a new nPod recommended package object
+
+        This constructor expects a ``dict`` object from the nebulon ON API. It
+        will check the returned data against the currently implemented schema
+        of the SDK.
+
+        :param response: The JSON response from the server
+        :type response: dict
+
+        :raises ValueError: An error if illegal data is returned from the
+            server.
+        """
+        self.__package_name = read_value(
+            "packageName", response, str, True)
+        self.__priority = read_value(
+            "priority", response, NebPackagePriority, True)
+        self.__offline = read_value(
+            "offline", response, bool, True)
+
+    @property
+    def package_name(self) -> str:
+        """The name of the nebulon update package"""
+        return self.__package_name
+
+    @property
+    def priority(self) -> NebPackagePriority:
+        """The importance for installing the recommended package"""
+        return self.__priority
+
+    @property
+    def offline(self) -> bool:
+        """Indicates if the installation requires stopping I/O"""
+        return self.__offline
+
+    @staticmethod
+    def fields():
+        return [
+            "packageName",
+            "priority",
+            "offline",
+        ]
+
+
 class RecommendedPackages:
     """Recommended packages for customers
 
@@ -191,14 +394,15 @@ class RecommendedPackages:
     ):
         """Constructs a new recommended packages object
 
-        This constructor expects a dict() object from the nebulon ON API. It
+        This constructor expects a ``dict`` object from the nebulon ON API. It
         will check the returned data against the currently implemented schema
         of the SDK.
 
         :param response: The JSON response from the server
         :type response: dict
 
-        :raises ValueError: An error if illegal data is returned from the server
+        :raises ValueError: An error if illegal data is returned from the
+            server.
         """
         self.__spu_type = read_value(
             "spuType", response, str, True)
@@ -226,7 +430,7 @@ class RecommendedPackages:
 
     @property
     def package_info(self) -> PackageInfo:
-        """Information concerning the update pacakge"""
+        """Information concerning the update package"""
         return self.__package_info
 
     @staticmethod
@@ -239,56 +443,68 @@ class RecommendedPackages:
         ]
 
 
-class UpdatePackages:
-    """An object describing nebOS software packages
+class PackageInfoList:
+    """Paginated software packages list
 
-    Describes software packages that are available for installation and
-    recommended for customers.
+    Contains a list of software package objects and information for
+    pagination. By default a single page includes a maximum of ``100`` items
+    unless specified otherwise in the paginated query.
+
+    Consumers should always check for the property ``more`` as per default
+    the server does not return the full list of alerts but only one page.
     """
-
     def __init__(
             self,
             response: dict
     ):
-        """Constructs a new update packages object
+        """Constructs a new update package list object
 
-        This constructor expects a dict() object from the nebulon ON API. It
+        This constructor expects a ``dict`` object from the nebulon ON API. It
         will check the returned data against the currently implemented schema
         of the SDK.
 
         :param response: The JSON response from the server
         :type response: dict
 
-        :raises ValueError: An error if illegal data is returned from the server
+        :raises ValueError: An error if illegal data is returned from the
+            server.
         """
-        self.__available = read_value(
-            "available", response, PackageInfo, False)
-        self.__recommended = read_value(
-            "recommended", response, RecommendedPackages, False)
-        self.__latest = read_value(
-            "latest", response, str, True)
+        self.__items = read_value(
+            "items", response, PackageInfo, True)
+        self.__more = read_value(
+            "more", response, bool, True)
+        self.__total_count = read_value(
+            "totalCount", response, int, True)
+        self.__filtered_count = read_value(
+            "filteredCount", response, int, True)
 
     @property
-    def available(self) -> [PackageInfo]:
-        """List of available nebulon software packages"""
-        return self.__available
+    def items(self) -> [PackageInfo]:
+        """List of software packages in the pagination list"""
+        return self.__items
 
     @property
-    def recommended(self) -> [RecommendedPackages]:
-        """List or recommended nebulon software packages"""
-        return self.__recommended
+    def more(self) -> bool:
+        """Indicates if there are more items on the server"""
+        return self.__more
 
     @property
-    def latest(self) -> str:
-        """The latest available nebulon software package"""
-        return self.__latest
+    def total_count(self) -> int:
+        """The total number of items on the server"""
+        return self.__total_count
+
+    @property
+    def filtered_count(self) -> int:
+        """The number of items on the server matching the provided filter"""
+        return self.__filtered_count
 
     @staticmethod
     def fields():
         return [
-            "available{%s}" % ",".join(PackageInfo.fields()),
-            "recommended{%s}" % ",".join(RecommendedPackages.fields()),
-            "latest",
+            "items{%s}" % ",".join(PackageInfo.fields()),
+            "more",
+            "totalCount",
+            "filteredCount",
         ]
 
 
@@ -301,14 +517,15 @@ class UpdateStateSpu:
     ):
         """Constructs a new update status object
 
-        This constructor expects a dict() object from the nebulon ON API. It
+        This constructor expects a ``dict`` object from the nebulon ON API. It
         will check the returned data against the currently implemented schema
         of the SDK.
 
         :param response: The JSON response from the server
         :type response: dict
 
-        :raises ValueError: An error if illegal data is returned from the server
+        :raises ValueError: An error if illegal data is returned from the
+            server.
         """
         self.__update_id = read_value(
             "updateID", response, str, True)
@@ -320,6 +537,8 @@ class UpdateStateSpu:
             "downloadProgressPct", response, int, True)
         self.__waiting_for_spu_serial = read_value(
             "waitingForSPUSerial", response, str, True)
+        self.__waiting_for_scheduled = read_value(
+            "waitingForScheduled", response, datetime, False)
         self.__started_install = read_value(
             "startedInstall", response, bool, True)
         self.__restarting = read_value(
@@ -356,6 +575,11 @@ class UpdateStateSpu:
         """Indicates if the SPU is waiting for a SPU to complete its update"""
         return self.__waiting_for_spu_serial
 
+    @property
+    def waiting_for_scheduled(self) -> datetime:
+        """Indicates that the SPU is waiting for a scheduled update"""
+        return self.__waiting_for_scheduled
+
     @property
     def started_install(self) -> bool:
         """Indicates if the SPU has started installing the SPU"""
@@ -389,6 +613,7 @@ class UpdateStateSpu:
             "packageName",
             "downloadProgressPct",
             "waitingForSPUSerial",
+            "waitingForScheduled",
             "startedInstall",
             "restarting",
             "restartComplete",
@@ -406,14 +631,15 @@ class UpdateHistory:
     ):
         """Constructs a new update history object
 
-        This constructor expects a dict() object from the nebulon ON API. It
+        This constructor expects a ``dict`` object from the nebulon ON API. It
         will check the returned data against the currently implemented schema
         of the SDK.
 
         :param response: The JSON response from the server
         :type response: dict
 
-        :raises ValueError: An error if illegal data is returned from the server
+        :raises ValueError: An error if illegal data is returned from the
+            server.
         """
         self.__update_id = read_value(
             "updateID", response, str, True)
@@ -448,7 +674,7 @@ class UpdateHistory:
 
     @property
     def success(self) -> bool:
-        """Indicates if the update completed successully"""
+        """Indicates if the update completed successfully"""
         return self.__success
 
     @staticmethod
@@ -465,24 +691,50 @@ class UpdateHistory:
 class UpdatesMixin(NebMixin):
     """Mixin to add update related methods to the GraphQL client"""
 
-    def get_update_packages(self) -> UpdatePackages:
+    def get_available_packages(
+            self,
+            page: PageInput = None,
+            available_packages_filter: AvailablePackagesFilter = None,
+            sort: AvailablePackagesSort = None
+    ) -> PackageInfoList:
         """Retrieves a list of update packages
 
-        :returns UpdatePackages: An object describing latest, available, and
-            recommended nebOS software packages
+        :param page: The requested page from the server. This is an optional
+            argument and if omitted the server will default to returning the
+            first page with a maximum of ``100`` items.
+        :type page: PageInput, optional
+        :param available_packages_filter: A filter object to filter the
+            software packages on the server. If omitted, the server will
+            return all objects as a paginated response.
+        :type available_packages_filter: AvailablePackagesFilter, optional
+        :param sort: A sort definition object to sort the software package
+            objects on supported properties. If omitted objects are
+            returned in the order as they were created in.
+        :type sort: AvailablePackagesSort, optional
+
+        :returns PackageInfoList: A paginated list of software packages
 
         :raises GraphQLError: An error with the GraphQL endpoint
         """
 
+        # setup query parameters
+        parameters = dict()
+        parameters["page"] = GraphQLParam(
+            page, "PageInput", False)
+        parameters["filter"] = GraphQLParam(
+            available_packages_filter, "AvailablePackagesFilter", False)
+        parameters["sort"] = GraphQLParam(
+            sort, "AvailablePackagesSort", False)
+
         # make the request
         response = self._query(
-            name="updatePackages",
-            params=None,
-            fields=UpdatePackages.fields()
+            name="getAvailablePackages",
+            params=parameters,
+            fields=PackageInfoList.fields()
         )
 
         # convert to object
-        return UpdatePackages(response)
+        return PackageInfoList(response)
 
     def __run_update_precheck(
             self,
@@ -534,32 +786,21 @@ class UpdatesMixin(NebMixin):
 
         :param npod_uuid: The unique identifier of the nPod to update
         :type npod_uuid: str
-        :param package_name: The package name to install
+        :param package_name: The name of the package to install
         :type package_name: str
         :param schedule_at: Allows scheduling the installation of a package
             at the specified date and time. If omitted, the package will be
             installed immediately
         :type schedule_at: datetime, optional
-        :param ignore_warnings: If specified warnings that are discovered during
-            the update pre-check are ignored (not recommended). If omitted or
-            set to ``False`` will cause the update to stop.
+        :param ignore_warnings: If specified warnings that are discovered
+            during the update pre-check are ignored (not recommended). If 
+            omitted or set to ``False`` will cause the update to stop.
         :type ignore_warnings: bool, optional
 
-        :raises GraphQLError: An error with the GraphQL endpoint.
-        :raises ValueError: If illegal parameters are specified
+        :raises GraphQLError: An error with the GraphQL endpoint
         :raises Exception: If token delivery fails
         """
 
-        # basic input validation
-        if npod_uuid is None or len(npod_uuid) == 0:
-            raise ValueError("npod_uuid must be a valid UUID string")
-
-        if package_name is None or len(package_name) == 0:
-            raise ValueError("package_name must be specified")
-
-        if schedule_at is not None and schedule_at < datetime.now():
-            raise ValueError("scheduled time may not be in the past")
-
         # first run an update pre-check
         issues = self.__run_update_precheck(
             npod_uuid=npod_uuid,
@@ -597,8 +838,8 @@ class UpdatesMixin(NebMixin):
         Allows querying for currently ongoing updates and their status
         information.
 
-        :param npod_uuid: Filter active updates for a specific nPod by providing
-            its unique identifier
+        :param npod_uuid: Filter active updates for a specific nPod by
+            providing its unique identifier
         :type npod_uuid: str
         :param update_uuid: The unique identifier of an ongoing update
         :type update_uuid: str
@@ -631,7 +872,10 @@ class UpdatesMixin(NebMixin):
             package_name: str,
             force: bool = False
     ):
-        """Update nebOS of an SPU to a specific package
+        """Update nebOS of a SPU to a specific package
+
+        This method is executed asynchronously and does not wait for the
+        update to complete.
 
         :param spu_serial: The serial number of the SPU to update
         :type spu_serial: str
@@ -664,7 +908,7 @@ class UpdatesMixin(NebMixin):
         token_response = TokenResponse(response)
         token_response.deliver_token()
 
-    def abort_spu_firmware(
+    def abort_update_spu_firmware(
             self,
             spu_serial: str = None,
             npod_uuid: str = None
