@@ -1,5 +1,5 @@
 #
-# Copyright 2020 Nebulon, Inc.
+# Copyright 2021 Nebulon, Inc.
 # All Rights Reserved.
 #
 # DISCLAIMER: THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND,
@@ -14,7 +14,7 @@
 
 from .graphqlclient import GraphQLParam, NebMixin
 from .common import PageInput, read_value
-from .filters import StringFilter, UuidFilter
+from .filters import StringFilter, UUIDFilter
 from .sorting import SortDirection
 
 
@@ -59,7 +59,7 @@ class WebHookFilter:
 
     def __init__(
             self,
-            uuid: UuidFilter = None,
+            uuid: UUIDFilter = None,
             name: StringFilter = None,
             url: StringFilter = None,
             enabled: bool = None,
@@ -73,7 +73,7 @@ class WebHookFilter:
         options to concatenate multiple filters.
 
         :param uuid: Filter based on webhook unique identifiers
-        :type uuid: UuidFilter, optional
+        :type uuid: UUIDFilter, optional
         :param name: Filter based on webhook name
         :type name: StringFilter, optional
         :param url: Filter based on webhook url
@@ -90,8 +90,8 @@ class WebHookFilter:
         self.__or = or_filter
 
     @property
-    def uuid(self) -> UuidFilter:
-        """Filter based on datacenter unique identifier"""
+    def uuid(self) -> UUIDFilter:
+        """Filter based on webhook unique identifier"""
         return self.__uuid
 
     @property
@@ -185,34 +185,34 @@ class CreateWebHookInput:
 
     Credentials can be specified in url field or headers field. Special
     variables can be used to specify credentials without storing them in
-    clear text. `$USERNAME`, `$PASSWORD`, `$AUTH_TOKEN` will be populated
+    clear text. ``$USERNAME``, ``$PASSWORD``, ``$AUTH_TOKEN`` will be populated
     during the execution of the webhook.
 
-    To identify open and close action for the alert, use `$ALERT_INCIDENT_ID`
+    To identify open and close action for the alert, use ``$ALERT_INCIDENT_ID``
     variable. The full list of available variables that are populated during
     webhook execution are:
 
-    * `$ALERT_INCIDENT_ID` - ID of the alert incident (open and close events)
-    * `$ALERT_EVENT_ID` - ID of open/close event for the alert
-    * `$ALERT_CODE` - Code of the alert
-    * `$ALERT_SUMMARY` - Summary of the alert status
-    * `$ALERT_STATUS` - status of the alert
-    * `$ALERT_DETAILS` - Full details of the alert
-    * `$ALERT_TIME` - Formatted time of when alert was created
-    * `$ALERT_URL` - URL of the alert, e.g.: https://on.nebulon.com/alert/{uuid}
-    * `$ALERT_SEVERITY` - Severity of the event (e.g. trivial, minor, etc.)
-    * `$RESOURCE_TYPE` - Type of the resource that alert is for
-    * `$RESOURCE_ID` - ID of the resource that alert is for
-    * `$RESOURCE_NAME` - Name of the resource that alert is for
-    * `$HOST_NAME` - Host name (if known) of the server in question
-    * `$HOST_SERIAL` - Host serial (if known) of the server in question
-    * `$HOST_MANUF` - Host manufacturer of the server in question
-    * `$SPU_SERIAL` - SPU serial number related to the described alert
-    * `$NPOD_UUID` - nPod UUID related to the described alert (if present)
-    * `$NPOD_NAME` - nPod name related to the described alert (if present)
-    * `$ORG_UUID` - Org UUID related to the alert
-    * `$ORG_NAME` - Organization name related to the alert
-    * `$CONTACT_EMAIL` - Email address of contact for the datacenter
+    * ``$ALERT_INCIDENT_ID`` - ID of the alert incident (open and close events)
+    * ``$ALERT_EVENT_ID`` - ID of open/close event for the alert
+    * ``$ALERT_CODE`` - Code of the alert
+    * ``$ALERT_SUMMARY`` - Summary of the alert status
+    * ``$ALERT_STATUS`` - status of the alert
+    * ``$ALERT_DETAILS`` - Full details of the alert
+    * ``$ALERT_TIME`` - Formatted time of when alert was created
+    * ``$ALERT_URL`` - URL of the alert, e.g.: https://on.nebulon.com/alert/{uuid}
+    * ``$ALERT_SEVERITY`` - Severity of the event (e.g. trivial, minor, etc.)
+    * ``$RESOURCE_TYPE`` - Type of the resource that alert is for
+    * ``$RESOURCE_ID`` - ID of the resource that alert is for
+    * ``$RESOURCE_NAME`` - Name of the resource that alert is for
+    * ``$HOST_NAME`` - Host name (if known) of the server in question
+    * ``$HOST_SERIAL`` - Host serial (if known) of the server in question
+    * ``$HOST_MANUF`` - Host manufacturer of the server in question
+    * ``$SPU_SERIAL`` - SPU serial number related to the described alert
+    * ``$NPOD_UUID`` - nPod UUID related to the described alert (if present)
+    * ``$NPOD_NAME`` - nPod name related to the described alert (if present)
+    * ``$ORG_UUID`` - Org UUID related to the alert
+    * ``$ORG_NAME`` - Organization name related to the alert
+    * ``$CONTACT_EMAIL`` - Email address of contact for the datacenter
     """
 
     def __init__(
@@ -253,25 +253,26 @@ class CreateWebHookInput:
         :type close_body: str, optional
         :param username: If username and password are used for authenticating
             with the target URL, use the username field. This will populate
-            the `$USERNAME` variable
+            the ``$USERNAME`` variable
         :type username: str, optional
         :param password: If username and password are used for authenticating
             with the target URL, use the password field. This will populate
-            the `$USERNAME` variable
+            the ``$USERNAME`` variable
         :type password: str, optional
         :param auth_token: If an authentication token is used with the webhook
-            use auth_token. This will populate the `$AUTH_TOKEN` variable.
+            use auth_token. This will populate the ``$AUTH_TOKEN`` variable.
         :type auth_token: str, optional
         :param headers: List of HTTP headers for the request
         :type headers: [HeaderInput], optional
         :param time_zone: The timezone to use when formatting time stings in
-            the webhook. If not specified
+            the webhook.
         :type time_zone: str, optional
         :param enabled: Allows specifying if the webhook shall be enabled or
             disabled after creation. If not specified, the webhook will be
             enabled by default.
         :type enabled: bool, optional
         """
+
         self.__name = name
         self.__open_url = open_url
         self.__close_url = close_url
@@ -301,17 +302,17 @@ class CreateWebHookInput:
 
     @property
     def username(self) -> str:
-        """Username field that populates the `$USERNAME` variable"""
+        """Username field that populates the ``$USERNAME`` variable"""
         return self.__username
 
     @property
     def password(self) -> str:
-        """Password field that populates the `$PASSWORD` variable"""
+        """Password field that populates the ``$PASSWORD`` variable"""
         return self.__password
 
     @property
     def auth_token(self) -> str:
-        """Auth token field that populates the `$AUTH_TOKEN` variable"""
+        """Auth token field that populates the ``$AUTH_TOKEN`` variable"""
         return self.__auth_token
 
     @property
@@ -356,7 +357,7 @@ class CreateWebHookInput:
 
 
 class UpdateWebHookInput:
-    """Input object for updating a new webhook
+    """Input object for updating a webhook
 
     Webhooks allow integration with notification services and workflow engines.
     When configured, webhooks are triggered for opened and closed alerts with
@@ -364,34 +365,34 @@ class UpdateWebHookInput:
 
     Credentials can be specified in url field or headers field. Special
     variables can be used to specify credentials without storing them in
-    clear text. `$USERNAME`, `$PASSWORD`, `$AUTH_TOKEN` will be populated
+    clear text. ``$USERNAME``, ``$PASSWORD``, ``$AUTH_TOKEN`` will be populated
     during the execution of the webhook.
 
-    To identify open and close action for the alert, use `$ALERT_INCIDENT_ID`
+    To identify open and close action for the alert, use ``$ALERT_INCIDENT_ID``
     variable. The full list of available variables that are populated during
     webhook execution are:
 
-    * `$ALERT_INCIDENT_ID` - ID of the alert incident (open and close events)
-    * `$ALERT_EVENT_ID` - ID of open/close event for the alert
-    * `$ALERT_CODE` - Code of the alert
-    * `$ALERT_SUMMARY` - Summary of the alert status
-    * `$ALERT_STATUS` - status of the alert
-    * `$ALERT_DETAILS` - Full details of the alert
-    * `$ALERT_TIME` - Formatted time of when alert was created
-    * `$ALERT_URL` - URL of the alert, e.g.: https://on.nebulon.com/alert/{uuid}
-    * `$ALERT_SEVERITY` - Severity of the event (e.g. trivial, minor, etc.)
-    * `$RESOURCE_TYPE` - Type of the resource that alert is for
-    * `$RESOURCE_ID` - ID of the resource that alert is for
-    * `$RESOURCE_NAME` - Name of the resource that alert is for
-    * `$HOST_NAME` - Host name (if known) of the server in question
-    * `$HOST_SERIAL` - Host serial (if known) of the server in question
-    * `$HOST_MANUF` - Host manufacturer of the server in question
-    * `$SPU_SERIAL` - SPU serial number related to the described alert
-    * `$NPOD_UUID` - nPod UUID related to the described alert (if present)
-    * `$NPOD_NAME` - nPod name related to the described alert (if present)
-    * `$ORG_UUID` - Org UUID related to the alert
-    * `$ORG_NAME` - Organization name related to the alert
-    * `$CONTACT_EMAIL` - Email address of contact for the datacenter
+    * ``$ALERT_INCIDENT_ID`` - ID of the alert incident (open and close events)
+    * ``$ALERT_EVENT_ID`` - ID of open/close event for the alert
+    * ``$ALERT_CODE`` - Code of the alert
+    * ``$ALERT_SUMMARY`` - Summary of the alert status
+    * ``$ALERT_STATUS`` - status of the alert
+    * ``$ALERT_DETAILS`` - Full details of the alert
+    * ``$ALERT_TIME`` - Formatted time of when alert was created
+    * ``$ALERT_URL`` - URL of the alert, e.g.: https://on.nebulon.com/alert/{uuid}
+    * ``$ALERT_SEVERITY`` - Severity of the event (e.g. trivial, minor, etc.)
+    * ``$RESOURCE_TYPE`` - Type of the resource that alert is for
+    * ``$RESOURCE_ID`` - ID of the resource that alert is for
+    * ``$RESOURCE_NAME`` - Name of the resource that alert is for
+    * ``$HOST_NAME`` - Host name (if known) of the server in question
+    * ``$HOST_SERIAL`` - Host serial (if known) of the server in question
+    * ``$HOST_MANUF`` - Host manufacturer of the server in question
+    * ``$SPU_SERIAL`` - SPU serial number related to the described alert
+    * ``$NPOD_UUID`` - nPod UUID related to the described alert (if present)
+    * ``$NPOD_NAME`` - nPod name related to the described alert (if present)
+    * ``$ORG_UUID`` - Org UUID related to the alert
+    * ``$ORG_NAME`` - Organization name related to the alert
+    * ``$CONTACT_EMAIL`` - Email address of contact for the datacenter
     """
 
     def __init__(
@@ -408,7 +409,7 @@ class UpdateWebHookInput:
             time_zone: str = None,
             enabled: bool = None
     ):
-        """Constructs an input object to update a new webhook
+        """Constructs an input object to update a webhook
 
         Webhooks allow integration with notification services and workflow
         engines. When configured, webhooks are triggered for opened and closed
@@ -432,23 +433,22 @@ class UpdateWebHookInput:
         :type close_body: str, optional
         :param username: If username and password are used for authenticating
             with the target URL, use the username field. This will populate
-            the `$USERNAME` variable
+            the ``$USERNAME`` variable
         :type username: str, optional
         :param password: If username and password are used for authenticating
             with the target URL, use the password field. This will populate
-            the `$USERNAME` variable
+            the ``$USERNAME`` variable
         :type password: str, optional
         :param auth_token: If an authentication token is used with the webhook
-            use auth_token. This will populate the `$AUTH_TOKEN` variable.
+            use auth_token. This will populate the ``$AUTH_TOKEN`` variable.
         :type auth_token: str, optional
         :param headers: List of HTTP headers for the request
         :type headers: [HeaderInput], optional
         :param time_zone: The timezone to use when formatting time stings in
-            the webhook. If not specified
+            the webhook.
         :type time_zone: str, optional
         :param enabled: Allows specifying if the webhook shall be enabled or
-            disabled after creation. If not specified, the webhook will be
-            enabled by default.
+            disabled.
         :type enabled: bool, optional
         """
 
@@ -481,17 +481,17 @@ class UpdateWebHookInput:
 
     @property
     def username(self) -> str:
-        """Username field that populates the `$USERNAME` variable"""
+        """Username field that populates the ``$USERNAME`` variable"""
         return self.__username
 
     @property
     def password(self) -> str:
-        """Password field that populates the `$PASSWORD` variable"""
+        """Password field that populates the ``$PASSWORD`` variable"""
         return self.__password
 
     @property
     def auth_token(self) -> str:
-        """Auth token field that populates the `$AUTH_TOKEN` variable"""
+        """Auth token field that populates the ``$AUTH_TOKEN`` variable"""
         return self.__auth_token
 
     @property
@@ -567,17 +567,17 @@ class TestWebHookInput:
         self.__update = update
 
     @property
-    def uuid(self):
+    def uuid(self) -> str:
         """The unique identifier of the webhook to be tested"""
         return self.__uuid
 
     @property
-    def create(self):
+    def create(self) -> CreateWebHookInput:
         """A definition for a new webhook to be tested"""
         return self.__create
 
     @property
-    def update(self):
+    def update(self) -> UpdateWebHookInput:
         """A definition for the updates for a webhook to be tested"""
         return self.__update
 
@@ -599,7 +599,7 @@ class TestWebHookResponse:
     ):
         """Constructs a new TestWebHookResponse object
 
-        This constructor expects a dict() object from the nebulon ON API. It
+        This constructor expects a ``dict`` object from the nebulon ON API. It
         will check the returned data against the currently implemented schema
         of the SDK.
 
@@ -622,12 +622,12 @@ class TestWebHookResponse:
 
     @property
     def open_error_message(self) -> str:
-        """Error message when testing the webhook with the ``open_url``"""
+        """Error message when testing the webhook with ``open_url``"""
         return self.__open_error_message
 
     @property
     def close_error_message(self) -> str:
-        """Error message when testing the webhook with the ``close_url``"""
+        """Error message when testing the webhook with ``close_url``"""
         return self.__close_error_message
 
     @staticmethod
@@ -648,7 +648,7 @@ class Header:
     ):
         """Constructs a new HTTP header object
 
-        This constructor expects a dict() object from the nebulon ON API. It
+        This constructor expects a ``dict`` object from the nebulon ON API. It
         will check the returned data against the currently implemented schema
         of the SDK.
 
@@ -694,7 +694,7 @@ class WebHook:
     ):
         """Constructs a new WebHook object
 
-        This constructor expects a dict() object from the nebulon ON API. It
+        This constructor expects a ``dict`` object from the nebulon ON API. It
         will check the returned data against the currently implemented schema
         of the SDK.
 
@@ -730,50 +730,62 @@ class WebHook:
 
     @property
     def uuid(self) -> str:
+        """The unique identifier of the webhook"""
         return self.__uuid
 
     @property
     def name(self) -> str:
+        """The human readable name for the webhook integration"""
         return self.__name
 
     @property
     def open_url(self) -> str:
+        """The URL that is called for alert open events"""
         return self.__open_url
 
     @property
     def close_url(self) -> str:
+        """The URL that is called for alert close events"""
         return self.__close_url
 
     @property
     def headers(self) -> list:
+        """HTTP headers that are used during webhook execution"""
         return self.__headers
 
     @property
     def open_body(self) -> str:
+        """The JSON-encoded body that is sent to the ``open_url`` URL"""
         return self.__open_body
 
     @property
     def close_body(self) -> str:
+        """The JSON-encoded body that is sent to the ``close_url`` URL"""
         return self.__close_body
 
     @property
     def enabled(self) -> bool:
+        """Indicates if the webhook is enabled or disabled"""
         return self.__enabled
 
     @property
     def time_zone(self) -> str:
+        """The time zone used for formatting date and time"""
         return self.__time_zone
 
     @property
     def errors(self) -> bool:
+        """Indicates if there were errors with the webhook execution"""
         return self.__errors
 
     @property
     def last_open_error(self) -> str:
+        """The last error message during execution of a webhook during alert open"""
         return self.__last_open_error
 
     @property
     def last_close_error(self) -> str:
+        """The last error message during execution of a webhook during alert close"""
         return self.__last_close_error
 
     @staticmethod
@@ -795,10 +807,10 @@ class WebHook:
 
 
 class WebHookList:
-    """Paginated webhook list object
+    """Paginated list of webhook objects
 
     Contains a list of webhook objects and information for
-    pagination. By default a single page includes a maximum of `100` items
+    pagination. By default a single page includes a maximum of ``100`` items
     unless specified otherwise in the paginated query.
 
     Consumers should always check for the property ``more`` as per default
@@ -811,7 +823,7 @@ class WebHookList:
     ):
         """Constructs a new webhook list object
 
-        This constructor expects a dict() object from the nebulon ON API. It
+        This constructor expects a ``dict`` object from the nebulon ON API. It
         will check the returned data against the currently implemented schema
         of the SDK.
 
@@ -830,7 +842,7 @@ class WebHookList:
             "filteredCount", response, int, True)
 
     @property
-    def items(self) -> list:
+    def items(self) -> [WebHook]:
         """List of webhooks in the pagination list"""
         return self.__items
 
@@ -865,19 +877,19 @@ class WebHookMixin(NebMixin):
     def get_webhooks(
             self,
             page: PageInput = None,
-            wh_filter: WebHookFilter = None,
+            webhook_filter: WebHookFilter = None,
             sort: WebHookSort = None,
     ) -> WebHookList:
         """Retrieves a list of webhooks
 
         :param page: The requested page from the server. This is an optional
             argument and if omitted the server will default to returning the
-            first page with a maximum of `100` items.
+            first page with a maximum of ``100`` items.
         :type page: PageInput, optional
-        :param wh_filter: A filter object to filter the webhooks on the
+        :param webhook_filter: A filter object to filter the webhooks on the
             server. If omitted, the server will return all objects as a
             paginated response.
-        :type wh_filter: WebHookFilter, optional
+        :type webhook_filter: WebHookFilter, optional
         :param sort: A sort definition object to sort the webhook objects
             on supported properties. If omitted objects are returned in the
             order as they were created in.
@@ -893,7 +905,7 @@ class WebHookMixin(NebMixin):
         parameters["page"] = GraphQLParam(
             page, "PageInput", False)
         parameters["filter"] = GraphQLParam(
-            wh_filter, "WebHookFilter", False)
+            webhook_filter, "WebHookFilter", False)
         parameters["sort"] = GraphQLParam(
             sort, "WebHookSort", False)
 
@@ -909,7 +921,7 @@ class WebHookMixin(NebMixin):
 
     def create_webhook(
             self,
-            definition: CreateWebHookInput
+            create_webhook_input: CreateWebHookInput
     ) -> WebHook:
         """Creates a new webhook
 
@@ -917,14 +929,18 @@ class WebHookMixin(NebMixin):
         engines. When configured, webhooks are triggered for opened and closed
         alerts with the specified webhook payload.
 
-        :param definition: The definition for a new webhook
-        :type definition: CreateWebHookInput
+        :param create_webhook_input: The definition for a new webhook
+        :type create_webhook_input: CreateWebHookInput
+
+        :returns WebHook: The created webhook.
+
+        :raises GraphQLError: An error with the GraphQL endpoint.
         """
 
         # setup query parameters
         parameters = dict()
         parameters["input"] = GraphQLParam(
-            definition,
+            create_webhook_input,
             "CreateWebHookInput",
             True
         )
@@ -942,7 +958,7 @@ class WebHookMixin(NebMixin):
     def update_webhook(
             self,
             uuid: str,
-            updates: UpdateWebHookInput
+            update_webhook_input: UpdateWebHookInput
     ) -> WebHook:
         """Updates an existing webhook
 
@@ -952,15 +968,20 @@ class WebHookMixin(NebMixin):
 
         :param uuid: The unique identifier of the webhook to update
         :type uuid: str
-        :param updates: The definition for updates to be made for the webhook
-        :type updates: UpdateWebHookInput
+        :param update_webhook_input: The definition for updates to be made for
+            the webhook
+        :type update_webhook_input: UpdateWebHookInput
+
+        :returns WebHook: The updated webhook.
+
+        :raises GraphQLError: An error with the GraphQL endpoint.
         """
 
         # setup query parameters
         parameters = dict()
         parameters["uuid"] = GraphQLParam(uuid, "UUID", True)
         parameters["input"] = GraphQLParam(
-            updates,
+            update_webhook_input,
             "UpdateWebHookInput",
             True
         )
@@ -983,6 +1004,10 @@ class WebHookMixin(NebMixin):
 
         :param uuid: The unique identifier of the webhook to delete
         :type uuid: str
+
+        :returns bool: If the deletion succeeded.
+
+        :raises GraphQLError: An error with the GraphQL endpoint.
         """
 
         # setup query parameters
@@ -1001,9 +1026,7 @@ class WebHookMixin(NebMixin):
 
     def test_webhook(
             self,
-            uuid: str = None,
-            create: CreateWebHookInput = None,
-            update: UpdateWebHookInput = None
+            test_webhook_input: TestWebHookInput
     ) -> TestWebHookResponse:
         """Test a webhook
 
@@ -1011,26 +1034,19 @@ class WebHookMixin(NebMixin):
         permanently stored in the cloud. When ``create`` is specified, neither
         ``uuid`` or ``update`` should be specified.
 
-        :param uuid: The unique identifier of an existing webhook that shall
-            be tested. Optionally, ``update`` can be specified when testing
-            changes to the existing webhook.
-        :type uuid: str, optional
-        :param create: The definition for a new webhook to be tested. If this
-            parameter is provided, neither ``uuid`` or ``update`` must be specified.
-        :type create: CreateWebHookInput, optional
-        :param update: The update definition for an existing webhook. If this
-            parameter is provided, also ``uuid`` is required.
-        :type update: UpdateWebHookInput, optional
+        :param test_webhook_input: An input object that defines the webhook
+            to be tested
+        :type test_webhook_input: TestWebHookInput
+
+        :returns TestWebHookResponse: The results of the test
+
+        :raises GraphQLError: An error with the GraphQL endpoint.
         """
 
         # setup query parameters
         parameters = dict()
         parameters["input"] = GraphQLParam(
-            TestWebHookInput(
-                uuid=uuid,
-                create=create,
-                update=update
-            ),
+            test_webhook_input,
             "TestWebHookInput",
             True)
 
